/*********************************************************************************
Copyright(c) 2020 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************/

/*!
* @file      ospi_flashdirectaccess_octal.c
*
* @brief     This is the primary header file for OSPI Octal Flash Direct read access example.
*
*/

#ifndef __OSPI_FLASHDIRECTACCESS_OCTAL_H__
#define __OSPI_FLASHDIRECTACCESS_OCTAL_H__


/*! This value defines the mask to enable/disable OSPI interrupts */
#define ADI_OSPI_INT_MASK                	0x00006030			  /* Enable STIG, WP, Ileggal access interrupts */
#define ADI_OSPI_DATAREAD_SETTING        	0x10021                /* Default sampling on rsing edge of SYSCLK */
#define ADI_OSPI_FLASH_AUTO_EXP_EN       	0
#define ADI_OSPI_FLASH_POLLING_COUNT     	1
#define ADI_OSPI_FLASH_POLLING_REP_DLY      0
#define ADI_OSPI_FLASH_POLLING_EXP          0xFFFFFFFF

#define BITM_OSPI_CTL_CS_LINE0              (_ADI_MSK_3(0x00000400,0x00000400UL, uint32_t  ))     /*  Periph Cs Line0 */


#define CONFIG_OCTALBAUD    (1)
///* Flash commands */
#define	FLASH_WR_EN					(0x06)  /*	Set Write Enable Latch */
#define FLASH_RD_STAT1				(0x05)  /*	Read Status Register1 */
#define FLASH_WR_WRCR2				(0x72)	/*	MX66LM1G write WRCR2 register */
#define FLASH_RD_WRCR2				(0x71)	/*	MX66LM1G read WRCR2 register */
#define WEL                  		(0x2)	/* Check the write enable bit of the SPI status register */

typedef struct
{
    volatile     	uint32_t   CONFIG_REG;                    /*!< Octal-SPI Configuration Register */
    volatile     	uint32_t   DEV_INSTR_RD_CONFIG_REG;       /*!< Device Read Instruction Configuration Register */
    volatile     	uint32_t   DEV_INSTR_WR_CONFIG_REG;       /*!< Device Write Instruction Configuration Register */
    volatile     	uint32_t   DEV_DELAY_REG;                 /*!< Dev Delay Reg */
    volatile     	uint32_t   RD_DATA_CAPTURE_REG;           /*!< Read Data Capture Register */
    volatile     	uint32_t   DEV_SIZE_CONFIG_REG;           /*!< Device Size Configuration Register */
    volatile const 	uint8_t    RESERVED0[12];
    volatile     	uint32_t   REMAP_ADDR_REG;                /*!< Remap Address Register */
    volatile     	uint32_t   MODE_BIT_CONFIG_REG;           /*!< Mode Bit Configuration Register */
    volatile const  uint8_t    RESERVED1[4];
    volatile     	uint32_t   TX_THRESH_REG;                 /*!< TX Threshold Register */
    volatile     	uint32_t   RX_THRESH_REG;                 /*!< RX Threshold Register */
    volatile     	uint32_t   WRITE_COMPLETION_CTRL_REG;     /*!< Write Completion Ctrl Reg */
    volatile     	uint32_t   NO_OF_POLLS_BEF_EXP_REG;       /*!< Polling Expiration Register */
    volatile     	uint32_t   IRQ_STATUS_REG;                /*!< Irq Status Reg */
    volatile     	uint32_t   IRQ_MASK_REG;                  /*!< Interrupt Mask:  0 : the interrupt for the corresponding interrupt status register bit is disabled. */
    volatile const  uint8_t    RESERVED2[8];
    volatile     	uint32_t   LOWER_WR_PROT_REG;             /*!< Lower Write Protection Register */
    volatile     	uint32_t   UPPER_WR_PROT_REG;             /*!< Upper Write Protection Register */
    volatile     	uint32_t   WR_PROT_CTRL_REG;              /*!< Write Protection Control Register */
    volatile const  uint8_t    RESERVED3[48];
    volatile     	uint32_t   FLASH_COMMAND_CTRL_MEM_REG;    /*!< Flash Command Control Memory Register */
    volatile     	uint32_t   FLASH_CMD_CTRL_REG;            /*!< Flash Command Control Register */
    volatile     	uint32_t   FLASH_CMD_ADDR_REG;            /*!< Flash Command Address Register */
    volatile const  uint8_t    RESERVED4[8];
    volatile const  uint32_t   FLASH_RD_DATA_LOWER_REG;       /*!< Flash Command Read Data Register (Lower) */
    volatile const  uint32_t   FLASH_RD_DATA_UPPER_REG;       /*!< Flash Command Read Data Register (Upper) */
    volatile     	uint32_t   FLASH_WR_DATA_LOWER_REG;       /*!< Flash Command Write Data Register (Lower) */
    volatile     	uint32_t   FLASH_WR_DATA_UPPER_REG;       /*!< Flash Command Write Data Register (Upper) */
    volatile     	uint32_t   POLLING_FLASH_STATUS_REG;      /*!< Polling Flash Status Register */
    volatile     	uint32_t   PHY_CONFIGURATION_REG;         /*!< PHY Configuration Register */
    volatile     	uint32_t   PHY_MASTER_CONTROL_REG;        /*!< PHY DLL Master Control Register */
    volatile const  uint32_t   DLL_OBSERVABLE_LOWER_REG;      /*!< DLL Observable Register Lower */
    volatile const  uint32_t   DLL_OBSERVABLE_UPPER_REG;      /*!< DLL Observable Register Upper */
    volatile const  uint8_t    RESERVED5[28];
    volatile     	uint32_t   OPCODE_EXT_LOWER_REG;          /*!< Opcode Extension Register (Lower) */
    volatile     	uint32_t   OPCODE_EXT_UPPER_REG;          /*!< Opcode Extension Register (Upper) */
    volatile const  uint8_t    RESERVED6[20];
    volatile const  uint32_t   MODULE_ID_REG;                 /*!< Module ID Register */
} ADI_OSPI_REGS;

typedef enum
{
    /*! Data transfer with Command, Address and data transffered in STR mode */
    ADI_OSPI_TRANSFERMODE_STR,

    /*! Data transfer with Command in STR mode, Address and data transffered in DTR mode */
    ADI_OSPI_TRANSFERMODE_DTR,

    /*! Data transfer with all Command, Address and data transffered in DTR mode */
    /*! To use this mode DDR should already be in DTR protocol mode */
    ADI_OSPI_TRANSFERMODE_DTR_P

} ADI_OSPI_TRANSFERMODE;

typedef enum
{
    /*! Data transfer with Command, Address and data on a single line */
    ADI_OSPI_STIG_SINGLE,

	/*! Data transfer with Command, Address and data on a Dual lines */
	ADI_OSPI_STIG_DPI,

    /*! Data transfer with Command, Address and data on a Quad lines */
    ADI_OSPI_STIG_QPI,

    /*! Data transfer with Command, Address and data on a Octal lines */
    ADI_OSPI_STIG_OPI

} ADI_OSPI_STIG_BUSMODE;

/*! Enumeration of Data mode requirment for STIG command.
 *
 * */
typedef enum
{
    /*! STIG command requires no data to read or write */
    ADI_OSPI_STIG_NODATA,

    /*! STIG command requires data to be read */
    ADI_OSPI_STIG_READDATA_EN,

     /*! STIG command will send data to flash */
    ADI_OSPI_STIG_WRITEDATA_EN

} ADI_OSPI_STIG_DATAMODE;

/*! Enumeration of command address sizes.
 *
 * */
typedef enum
{
    /*! 1 address byte */
    ADI_OSPI_CMD_ADDR_SIZE_1 = 1,
    /*! 2 address bytes */
    ADI_OSPI_CMD_ADDR_SIZE_2,
    /*! 3 address bytes */
    ADI_OSPI_CMD_ADDR_SIZE_3,
    /*! 4 address bytes */
    ADI_OSPI_CMD_ADDR_SIZE_4
} ADI_OSPI_CMDADDRSIZE;

/**
 * \struct ADI_OSPI_STIGCMD
 * Defines different inputs for the command to be executed using STIG mechanism.
 *
 * */
typedef struct
{
    /*! command op-code */
    uint8_t CmdOpcode;
    /*! 2nd command byte for commands with extended op-code,
        ! Must be st to '0' for all normal commands */
    uint8_t CmdOpcode2;

    /*! Bus mode */
    ADI_OSPI_STIG_BUSMODE eBusMode;

    /*! Transfer mode */
    ADI_OSPI_TRANSFERMODE eTransferMode;

    /*! Number of dummy bytes */
    uint8_t NumOfDummyCycles;

    /*! Whether Mode data will be sent */
    uint32_t ModeDataEnable:1;

    /*! Mode byte data */
    uint8_t ModeData;

    /*! Specifies whether the command has any data requirements */
    ADI_OSPI_STIG_DATAMODE eDataMode;

    /*! Number of Data Bytes */
    uint16_t DataSize;

    /*! address will be sent */
    uint32_t AddressEnable:1;

    /*! Number of Address Bytes */
    ADI_OSPI_CMDADDRSIZE AddrSize;

    /*! flash command address */
    uint32_t Address;
     /*! Pointer to the data buffer.
      * Acts as write buffer for commands which send out data
	  * Acts as read buffer for commands which expect data in response
      */
    void *pBuffer;
} ADI_OSPI_STIGCMD;



/* Octal STR/DTR mode selection enum for Flash device */
typedef enum
{
    /*! Flash is configured in STR OPI mode */
	STR_MODE,

    /*!  Flash is configured in DTR OPI mode */
	DTR_MODE,

} FLASH_OPI_MODE;




/* Function prototypes */
int Configure_Flash(ADI_OSPI_STIGCMD *pSTIG_Command);
int  OPI_En_MX66LM1G(ADI_OSPI_STIGCMD *pSTIG_Command, FLASH_OPI_MODE Transfer_Mode);
void adi_ospi_StigCommand(ADI_OSPI_STIGCMD *pStigCommand);
void adi_ospi_configure_Stigbusmode(ADI_OSPI_STIGCMD * pStigCommand);
void adi_ospi_configure_TransferMode(ADI_OSPI_TRANSFERMODE eTransferMode);
void Configure_Octal(void);

#endif /* __OSPI_FLASHDIRECTACCESS_OCTAL_H__ */
